@@ -177,95 +177,105 @@ extern int yydebug;
     VAR_FORWARD_FIRST = 387,
     VAR_STUB_SSL_UPSTREAM = 388,
     VAR_FORWARD_SSL_UPSTREAM = 389,
-    VAR_STUB_FIRST = 390,
-    VAR_MINIMAL_RESPONSES = 391,
-    VAR_RRSET_ROUNDROBIN = 392,
-    VAR_MAX_UDP_SIZE = 393,
-    VAR_DELAY_CLOSE = 394,
-    VAR_UNBLOCK_LAN_ZONES = 395,
-    VAR_INSECURE_LAN_ZONES = 396,
-    VAR_INFRA_CACHE_MIN_RTT = 397,
-    VAR_DNS64_PREFIX = 398,
-    VAR_DNS64_SYNTHALL = 399,
-    VAR_DNSTAP = 400,
-    VAR_DNSTAP_ENABLE = 401,
-    VAR_DNSTAP_SOCKET_PATH = 402,
-    VAR_DNSTAP_SEND_IDENTITY = 403,
-    VAR_DNSTAP_SEND_VERSION = 404,
-    VAR_DNSTAP_IDENTITY = 405,
-    VAR_DNSTAP_VERSION = 406,
-    VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES = 407,
-    VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES = 408,
-    VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES = 409,
-    VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES = 410,
-    VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES = 411,
-    VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES = 412,
-    VAR_RESPONSE_IP_TAG = 413,
-    VAR_RESPONSE_IP = 414,
-    VAR_RESPONSE_IP_DATA = 415,
-    VAR_HARDEN_ALGO_DOWNGRADE = 416,
-    VAR_IP_TRANSPARENT = 417,
-    VAR_DISABLE_DNSSEC_LAME_CHECK = 418,
-    VAR_IP_RATELIMIT = 419,
-    VAR_IP_RATELIMIT_SLABS = 420,
-    VAR_IP_RATELIMIT_SIZE = 421,
-    VAR_RATELIMIT = 422,
-    VAR_RATELIMIT_SLABS = 423,
-    VAR_RATELIMIT_SIZE = 424,
-    VAR_RATELIMIT_FOR_DOMAIN = 425,
-    VAR_RATELIMIT_BELOW_DOMAIN = 426,
-    VAR_IP_RATELIMIT_FACTOR = 427,
-    VAR_RATELIMIT_FACTOR = 428,
-    VAR_SEND_CLIENT_SUBNET = 429,
-    VAR_CLIENT_SUBNET_ZONE = 430,
-    VAR_CLIENT_SUBNET_ALWAYS_FORWARD = 431,
-    VAR_CLIENT_SUBNET_OPCODE = 432,
-    VAR_MAX_CLIENT_SUBNET_IPV4 = 433,
-    VAR_MAX_CLIENT_SUBNET_IPV6 = 434,
-    VAR_CAPS_WHITELIST = 435,
-    VAR_CACHE_MAX_NEGATIVE_TTL = 436,
-    VAR_PERMIT_SMALL_HOLDDOWN = 437,
-    VAR_QNAME_MINIMISATION = 438,
-    VAR_QNAME_MINIMISATION_STRICT = 439,
-    VAR_IP_FREEBIND = 440,
-    VAR_DEFINE_TAG = 441,
-    VAR_LOCAL_ZONE_TAG = 442,
-    VAR_ACCESS_CONTROL_TAG = 443,
-    VAR_LOCAL_ZONE_OVERRIDE = 444,
-    VAR_ACCESS_CONTROL_TAG_ACTION = 445,
-    VAR_ACCESS_CONTROL_TAG_DATA = 446,
-    VAR_VIEW = 447,
-    VAR_ACCESS_CONTROL_VIEW = 448,
-    VAR_VIEW_FIRST = 449,
-    VAR_SERVE_EXPIRED = 450,
-    VAR_FAKE_DSA = 451,
-    VAR_FAKE_SHA1 = 452,
-    VAR_LOG_IDENTITY = 453,
-    VAR_HIDE_TRUSTANCHOR = 454,
-    VAR_TRUST_ANCHOR_SIGNALING = 455,
-    VAR_USE_SYSTEMD = 456,
-    VAR_SHM_ENABLE = 457,
-    VAR_SHM_KEY = 458,
-    VAR_DNSCRYPT = 459,
-    VAR_DNSCRYPT_ENABLE = 460,
-    VAR_DNSCRYPT_PORT = 461,
-    VAR_DNSCRYPT_PROVIDER = 462,
-    VAR_DNSCRYPT_SECRET_KEY = 463,
-    VAR_DNSCRYPT_PROVIDER_CERT = 464,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 465,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 466,
-    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 467,
-    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 468,
-    VAR_IPSECMOD_ENABLED = 469,
-    VAR_IPSECMOD_HOOK = 470,
-    VAR_IPSECMOD_IGNORE_BOGUS = 471,
-    VAR_IPSECMOD_MAX_TTL = 472,
-    VAR_IPSECMOD_WHITELIST = 473,
-    VAR_IPSECMOD_STRICT = 474,
-    VAR_CACHEDB = 475,
-    VAR_CACHEDB_BACKEND = 476,
-    VAR_CACHEDB_SECRETSEED = 477,
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 478
+    VAR_TLS_CERT_BUNDLE = 390,
+    VAR_STUB_FIRST = 391,
+    VAR_MINIMAL_RESPONSES = 392,
+    VAR_RRSET_ROUNDROBIN = 393,
+    VAR_MAX_UDP_SIZE = 394,
+    VAR_DELAY_CLOSE = 395,
+    VAR_UNBLOCK_LAN_ZONES = 396,
+    VAR_INSECURE_LAN_ZONES = 397,
+    VAR_INFRA_CACHE_MIN_RTT = 398,
+    VAR_DNS64_PREFIX = 399,
+    VAR_DNS64_SYNTHALL = 400,
+    VAR_DNSTAP = 401,
+    VAR_DNSTAP_ENABLE = 402,
+    VAR_DNSTAP_SOCKET_PATH = 403,
+    VAR_DNSTAP_SEND_IDENTITY = 404,
+    VAR_DNSTAP_SEND_VERSION = 405,
+    VAR_DNSTAP_IDENTITY = 406,
+    VAR_DNSTAP_VERSION = 407,
+    VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES = 408,
+    VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES = 409,
+    VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES = 410,
+    VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES = 411,
+    VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES = 412,
+    VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES = 413,
+    VAR_RESPONSE_IP_TAG = 414,
+    VAR_RESPONSE_IP = 415,
+    VAR_RESPONSE_IP_DATA = 416,
+    VAR_HARDEN_ALGO_DOWNGRADE = 417,
+    VAR_IP_TRANSPARENT = 418,
+    VAR_DISABLE_DNSSEC_LAME_CHECK = 419,
+    VAR_IP_RATELIMIT = 420,
+    VAR_IP_RATELIMIT_SLABS = 421,
+    VAR_IP_RATELIMIT_SIZE = 422,
+    VAR_RATELIMIT = 423,
+    VAR_RATELIMIT_SLABS = 424,
+    VAR_RATELIMIT_SIZE = 425,
+    VAR_RATELIMIT_FOR_DOMAIN = 426,
+    VAR_RATELIMIT_BELOW_DOMAIN = 427,
+    VAR_IP_RATELIMIT_FACTOR = 428,
+    VAR_RATELIMIT_FACTOR = 429,
+    VAR_SEND_CLIENT_SUBNET = 430,
+    VAR_CLIENT_SUBNET_ZONE = 431,
+    VAR_CLIENT_SUBNET_ALWAYS_FORWARD = 432,
+    VAR_CLIENT_SUBNET_OPCODE = 433,
+    VAR_MAX_CLIENT_SUBNET_IPV4 = 434,
+    VAR_MAX_CLIENT_SUBNET_IPV6 = 435,
+    VAR_CAPS_WHITELIST = 436,
+    VAR_CACHE_MAX_NEGATIVE_TTL = 437,
+    VAR_PERMIT_SMALL_HOLDDOWN = 438,
+    VAR_QNAME_MINIMISATION = 439,
+    VAR_QNAME_MINIMISATION_STRICT = 440,
+    VAR_IP_FREEBIND = 441,
+    VAR_DEFINE_TAG = 442,
+    VAR_LOCAL_ZONE_TAG = 443,
+    VAR_ACCESS_CONTROL_TAG = 444,
+    VAR_LOCAL_ZONE_OVERRIDE = 445,
+    VAR_ACCESS_CONTROL_TAG_ACTION = 446,
+    VAR_ACCESS_CONTROL_TAG_DATA = 447,
+    VAR_VIEW = 448,
+    VAR_ACCESS_CONTROL_VIEW = 449,
+    VAR_VIEW_FIRST = 450,
+    VAR_SERVE_EXPIRED = 451,
+    VAR_FAKE_DSA = 452,
+    VAR_FAKE_SHA1 = 453,
+    VAR_LOG_IDENTITY = 454,
+    VAR_HIDE_TRUSTANCHOR = 455,
+    VAR_TRUST_ANCHOR_SIGNALING = 456,
+    VAR_AGGRESSIVE_NSEC = 457,
+    VAR_USE_SYSTEMD = 458,
+    VAR_SHM_ENABLE = 459,
+    VAR_SHM_KEY = 460,
+    VAR_DNSCRYPT = 461,
+    VAR_DNSCRYPT_ENABLE = 462,
+    VAR_DNSCRYPT_PORT = 463,
+    VAR_DNSCRYPT_PROVIDER = 464,
+    VAR_DNSCRYPT_SECRET_KEY = 465,
+    VAR_DNSCRYPT_PROVIDER_CERT = 466,
+    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 467,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 468,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 469,
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 470,
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 471,
+    VAR_IPSECMOD_ENABLED = 472,
+    VAR_IPSECMOD_HOOK = 473,
+    VAR_IPSECMOD_IGNORE_BOGUS = 474,
+    VAR_IPSECMOD_MAX_TTL = 475,
+    VAR_IPSECMOD_WHITELIST = 476,
+    VAR_IPSECMOD_STRICT = 477,
+    VAR_CACHEDB = 478,
+    VAR_CACHEDB_BACKEND = 479,
+    VAR_CACHEDB_SECRETSEED = 480,
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 481,
+    VAR_FOR_UPSTREAM = 482,
+    VAR_AUTH_ZONE = 483,
+    VAR_ZONEFILE = 484,
+    VAR_MASTER = 485,
+    VAR_URL = 486,
+    VAR_FOR_DOWNSTREAM = 487,
+    VAR_FALLBACK_ENABLED = 488
   };
 #endif
 /* Tokens.  */
@@ -401,95 +411,105 @@ extern int yydebug;
 #define VAR_FORWARD_FIRST 387
 #define VAR_STUB_SSL_UPSTREAM 388
 #define VAR_FORWARD_SSL_UPSTREAM 389
-#define VAR_STUB_FIRST 390
-#define VAR_MINIMAL_RESPONSES 391
-#define VAR_RRSET_ROUNDROBIN 392
-#define VAR_MAX_UDP_SIZE 393
-#define VAR_DELAY_CLOSE 394
-#define VAR_UNBLOCK_LAN_ZONES 395
-#define VAR_INSECURE_LAN_ZONES 396
-#define VAR_INFRA_CACHE_MIN_RTT 397
-#define VAR_DNS64_PREFIX 398
-#define VAR_DNS64_SYNTHALL 399
-#define VAR_DNSTAP 400
-#define VAR_DNSTAP_ENABLE 401
-#define VAR_DNSTAP_SOCKET_PATH 402
-#define VAR_DNSTAP_SEND_IDENTITY 403
-#define VAR_DNSTAP_SEND_VERSION 404
-#define VAR_DNSTAP_IDENTITY 405
-#define VAR_DNSTAP_VERSION 406
-#define VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES 407
-#define VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES 408
-#define VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES 409
-#define VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES 410
-#define VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES 411
-#define VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES 412
-#define VAR_RESPONSE_IP_TAG 413
-#define VAR_RESPONSE_IP 414
-#define VAR_RESPONSE_IP_DATA 415
-#define VAR_HARDEN_ALGO_DOWNGRADE 416
-#define VAR_IP_TRANSPARENT 417
-#define VAR_DISABLE_DNSSEC_LAME_CHECK 418
-#define VAR_IP_RATELIMIT 419
-#define VAR_IP_RATELIMIT_SLABS 420
-#define VAR_IP_RATELIMIT_SIZE 421
-#define VAR_RATELIMIT 422
-#define VAR_RATELIMIT_SLABS 423
-#define VAR_RATELIMIT_SIZE 424
-#define VAR_RATELIMIT_FOR_DOMAIN 425
-#define VAR_RATELIMIT_BELOW_DOMAIN 426
-#define VAR_IP_RATELIMIT_FACTOR 427
-#define VAR_RATELIMIT_FACTOR 428
-#define VAR_SEND_CLIENT_SUBNET 429
-#define VAR_CLIENT_SUBNET_ZONE 430
-#define VAR_CLIENT_SUBNET_ALWAYS_FORWARD 431
-#define VAR_CLIENT_SUBNET_OPCODE 432
-#define VAR_MAX_CLIENT_SUBNET_IPV4 433
-#define VAR_MAX_CLIENT_SUBNET_IPV6 434
-#define VAR_CAPS_WHITELIST 435
-#define VAR_CACHE_MAX_NEGATIVE_TTL 436
-#define VAR_PERMIT_SMALL_HOLDDOWN 437
-#define VAR_QNAME_MINIMISATION 438
-#define VAR_QNAME_MINIMISATION_STRICT 439
-#define VAR_IP_FREEBIND 440
-#define VAR_DEFINE_TAG 441
-#define VAR_LOCAL_ZONE_TAG 442
-#define VAR_ACCESS_CONTROL_TAG 443
-#define VAR_LOCAL_ZONE_OVERRIDE 444
-#define VAR_ACCESS_CONTROL_TAG_ACTION 445
-#define VAR_ACCESS_CONTROL_TAG_DATA 446
-#define VAR_VIEW 447
-#define VAR_ACCESS_CONTROL_VIEW 448
-#define VAR_VIEW_FIRST 449
-#define VAR_SERVE_EXPIRED 450
-#define VAR_FAKE_DSA 451
-#define VAR_FAKE_SHA1 452
-#define VAR_LOG_IDENTITY 453
-#define VAR_HIDE_TRUSTANCHOR 454
-#define VAR_TRUST_ANCHOR_SIGNALING 455
-#define VAR_USE_SYSTEMD 456
-#define VAR_SHM_ENABLE 457
-#define VAR_SHM_KEY 458
-#define VAR_DNSCRYPT 459
-#define VAR_DNSCRYPT_ENABLE 460
-#define VAR_DNSCRYPT_PORT 461
-#define VAR_DNSCRYPT_PROVIDER 462
-#define VAR_DNSCRYPT_SECRET_KEY 463
-#define VAR_DNSCRYPT_PROVIDER_CERT 464
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 465
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 466
-#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 467
-#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 468
-#define VAR_IPSECMOD_ENABLED 469
-#define VAR_IPSECMOD_HOOK 470
-#define VAR_IPSECMOD_IGNORE_BOGUS 471
-#define VAR_IPSECMOD_MAX_TTL 472
-#define VAR_IPSECMOD_WHITELIST 473
-#define VAR_IPSECMOD_STRICT 474
-#define VAR_CACHEDB 475
-#define VAR_CACHEDB_BACKEND 476
-#define VAR_CACHEDB_SECRETSEED 477
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 478
+#define VAR_TLS_CERT_BUNDLE 390
+#define VAR_STUB_FIRST 391
+#define VAR_MINIMAL_RESPONSES 392
+#define VAR_RRSET_ROUNDROBIN 393
+#define VAR_MAX_UDP_SIZE 394
+#define VAR_DELAY_CLOSE 395
+#define VAR_UNBLOCK_LAN_ZONES 396
+#define VAR_INSECURE_LAN_ZONES 397
+#define VAR_INFRA_CACHE_MIN_RTT 398
+#define VAR_DNS64_PREFIX 399
+#define VAR_DNS64_SYNTHALL 400
+#define VAR_DNSTAP 401
+#define VAR_DNSTAP_ENABLE 402
+#define VAR_DNSTAP_SOCKET_PATH 403
+#define VAR_DNSTAP_SEND_IDENTITY 404
+#define VAR_DNSTAP_SEND_VERSION 405
+#define VAR_DNSTAP_IDENTITY 406
+#define VAR_DNSTAP_VERSION 407
+#define VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES 408
+#define VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES 409
+#define VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES 410
+#define VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES 411
+#define VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES 412
+#define VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES 413
+#define VAR_RESPONSE_IP_TAG 414
+#define VAR_RESPONSE_IP 415
+#define VAR_RESPONSE_IP_DATA 416
+#define VAR_HARDEN_ALGO_DOWNGRADE 417
+#define VAR_IP_TRANSPARENT 418
+#define VAR_DISABLE_DNSSEC_LAME_CHECK 419
+#define VAR_IP_RATELIMIT 420
+#define VAR_IP_RATELIMIT_SLABS 421
+#define VAR_IP_RATELIMIT_SIZE 422
+#define VAR_RATELIMIT 423
+#define VAR_RATELIMIT_SLABS 424
+#define VAR_RATELIMIT_SIZE 425
+#define VAR_RATELIMIT_FOR_DOMAIN 426
+#define VAR_RATELIMIT_BELOW_DOMAIN 427
+#define VAR_IP_RATELIMIT_FACTOR 428
+#define VAR_RATELIMIT_FACTOR 429
+#define VAR_SEND_CLIENT_SUBNET 430
+#define VAR_CLIENT_SUBNET_ZONE 431
+#define VAR_CLIENT_SUBNET_ALWAYS_FORWARD 432
+#define VAR_CLIENT_SUBNET_OPCODE 433
+#define VAR_MAX_CLIENT_SUBNET_IPV4 434
+#define VAR_MAX_CLIENT_SUBNET_IPV6 435
+#define VAR_CAPS_WHITELIST 436
+#define VAR_CACHE_MAX_NEGATIVE_TTL 437
+#define VAR_PERMIT_SMALL_HOLDDOWN 438
+#define VAR_QNAME_MINIMISATION 439
+#define VAR_QNAME_MINIMISATION_STRICT 440
+#define VAR_IP_FREEBIND 441
+#define VAR_DEFINE_TAG 442
+#define VAR_LOCAL_ZONE_TAG 443
+#define VAR_ACCESS_CONTROL_TAG 444
+#define VAR_LOCAL_ZONE_OVERRIDE 445
+#define VAR_ACCESS_CONTROL_TAG_ACTION 446
+#define VAR_ACCESS_CONTROL_TAG_DATA 447
+#define VAR_VIEW 448
+#define VAR_ACCESS_CONTROL_VIEW 449
+#define VAR_VIEW_FIRST 450
+#define VAR_SERVE_EXPIRED 451
+#define VAR_FAKE_DSA 452
+#define VAR_FAKE_SHA1 453
+#define VAR_LOG_IDENTITY 454
+#define VAR_HIDE_TRUSTANCHOR 455
+#define VAR_TRUST_ANCHOR_SIGNALING 456
+#define VAR_AGGRESSIVE_NSEC 457
+#define VAR_USE_SYSTEMD 458
+#define VAR_SHM_ENABLE 459
+#define VAR_SHM_KEY 460
+#define VAR_DNSCRYPT 461
+#define VAR_DNSCRYPT_ENABLE 462
+#define VAR_DNSCRYPT_PORT 463
+#define VAR_DNSCRYPT_PROVIDER 464
+#define VAR_DNSCRYPT_SECRET_KEY 465
+#define VAR_DNSCRYPT_PROVIDER_CERT 466
+#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 467
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 468
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 469
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 470
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 471
+#define VAR_IPSECMOD_ENABLED 472
+#define VAR_IPSECMOD_HOOK 473
+#define VAR_IPSECMOD_IGNORE_BOGUS 474
+#define VAR_IPSECMOD_MAX_TTL 475
+#define VAR_IPSECMOD_WHITELIST 476
+#define VAR_IPSECMOD_STRICT 477
+#define VAR_CACHEDB 478
+#define VAR_CACHEDB_BACKEND 479
+#define VAR_CACHEDB_SECRETSEED 480
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 481
+#define VAR_FOR_UPSTREAM 482
+#define VAR_AUTH_ZONE 483
+#define VAR_ZONEFILE 484
+#define VAR_MASTER 485
+#define VAR_URL 486
+#define VAR_FOR_DOWNSTREAM 487
+#define VAR_FALLBACK_ENABLED 488
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -500,7 +520,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 504 "util/configparser.h" /* yacc.c:1909  */
+#line 524 "util/configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
